#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import signal
import subprocess
from shutil import which
from optparse import OptionParser
import trtc_folder_parser
import trtc_convert

if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal.SIG_IGN)
    signal.signal(signal.SIGQUIT, signal.SIG_IGN)

    cwd = os.getcwd()
    os.chdir(os.path.dirname(os.path.realpath(__file__)))
    if not os.path.exists('./ffmpeg'):
        if which('unzip') is None:
            print("please install unzip or unzip ffmpeg.zip by yourself.")
            exit
        subprocess.run(['unzip', 'ffmpeg.zip'])
    os.chdir(cwd)

    parser = OptionParser()
    parser.add_option(
        "-f",
        "--folder",
        type="string",
        dest="folder",
        help="Folder of files to be merged",
        default="",
    )
    parser.add_option(
        "-m",
        "--mode",
        type="int",
        dest="mode",
        help="Convert merge mode.\n[0: segment merge A/V(Default);\n 1: uid merge A/V]",
        default=0,
    )
    parser.add_option(
        "-p",
        "--fps",
        type="int",
        dest="fps",
        help="Convert fps, default 15",
        default=15,
    )
    parser.add_option(
        "-s",
        "--saving",
        action="store_true",
        dest="saving",
        help="Convert Do not time sync",
        default=False,
    )
    parser.add_option(
        "-a",
        "--aux_use",
        type="int",
        dest="aux_use",
        help="aux stream mode",
        default=False,
    )
    parser.add_option(
        "-r",
        "--resolution",
        type="int",
        dest="resolution",
        nargs=2,
        help="Specific resolution to convert '-r width height' \nEg:'-r 640 360'",
        default=(0, 0),
    )
    parser.add_option(
        "-b",
        "--fill_black",
        action="store_true",
        dest="fill_black",
        help="Show black frame when there is no video file",
        default=False,
    )

    (options, args) = parser.parse_args()
    if not options.folder:
        parser.print_help()
        parser.error("Not set folder")

    options.folder = os.path.abspath(options.folder)
    os.environ["FPSARG"] = "%s" % options.fps
    os.environ["USEAUX"] = "%d" % options.aux_use
    os.environ["MERGEMODE"] = "%d" % options.mode
    os.environ["MERGEGAP"] = "%d" % options.saving
    Parser = trtc_folder_parser.TrtcFloderParser(options.folder)
    Parser.process()
    trtc_convert.start_convert(options)
    Parser.clean()
